<?php

namespace Drupal\crossword_image;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\crossword\CrosswordFileParserManager;
use Drupal\file\FileInterface;
use Drupal\Core\File\FileSystem;
use Drupal\Core\ImageToolkit\ImageToolkitManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A base class for plugins that create images from crossword files.
 */
abstract class CrosswordImagePluginBase extends PluginBase implements CrosswordImagePluginInterface, ContainerFactoryPluginInterface {

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystem
   */
  protected $fileSystem;

  /**
   * The crossword file parser plugin manage.
   *
   * @var \Drupal\crossword\CrosswordFileParserManager
   */
  protected $parserManager;

  /**
   * The image toolkit plugin manage.
   *
   * @var \Drupal\Core\ImageToolkit\ImageToolkitManager
   */
  protected $toolkitManager;

  /**
   * The image toolkit plugin.
   *
   * @var \Drupal\Core\ImageToolkit\ImageToolkitInterface
   */
  protected $toolkit;

  /**
   * Construct the Crossword Image Plugin.
   */
  public function __construct($configuration, $plugin_id, array $plugin_definition, FileSystem $file_system, CrosswordFileParserManager $parser_manager, ImageToolkitManager $toolkit_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->fileSystem = $file_system;
    $this->parserManager = $parser_manager;
    $this->toolkitManager = $toolkit_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('file_system'),
      $container->get('crossword.manager.parser'),
      $container->get('image.toolkit.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getImageUri(FileInterface $file) {
    $destination_uri = $this->getDestinationUri($file);
    if (file_exists($destination_uri)) {
      // Check if the existing image is older than the file itself.
      if (filemtime($file->getFileUri()) <= filemtime($destination_uri)) {
        // The existing image can be used, nothing to do.
        return $destination_uri;
      }
      else {
        // Delete the existing but out-of-date image.
        $this->fileSystem->delete($destination_uri);
        image_path_flush($destination_uri);
      }
    }
    if ($this->saveNewImageFile($file, $destination_uri)) {
      return $destination_uri;
    }
  }

  /**
   * Creates and saves image representation of the crossword file.
   *
   * @param \Drupal\file\FileInterface $file
   *   The file from which the image is to be created.
   * @param string $destination_uri
   *   The destination of the new file.
   *
   * @return bool
   *   Returns TRUE upon success, FALSE upon failure.
   */
  protected function saveNewImageFile(FileInterface $file, $destination_uri) {
    $parser = $this->parserManager->loadCrosswordFileParserFromInput($file);
    if (!$parser) {
      return NULL;
    }
    $data = $parser->parse();
    $this->toolkit = $this->toolkitManager->createInstance($this->getPluginDefinition()['toolkit']);
    $this->toolkit->setType($this->getType());
    $this->toolkit->setResource($this->createImageResource($data));
    $directory = $this->fileSystem->dirname($destination_uri);
    $this->fileSystem->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY);
    return $this->toolkit->save($destination_uri);
  }

  /**
   * Gets the destination URI of the file.
   *
   * By default the image is saved in a subdirectory of the directory that
   * holds the crossword file itself.
   *
   * @param \Drupal\file\FileInterface $file
   *   The file that is being converted.
   *
   * @return string
   *   The destination URI.
   */
  protected function getDestinationUri(FileInterface $file) {
    $file_dir = dirname($file->getFileUri());
    if ($file_dir == 'public:') {
      $output_path = 'public://crossword';
    }
    else {
      $output_path = $file_dir . '/crossword';
    }
    $extension = image_type_to_extension($this->getType(), FALSE);
    $filename = "{$file->id()}-{$this->getPluginId()}.$extension";
    return $output_path . '/' . $filename;
  }

  /**
   * The type of image to be saved, as given in annotation.
   *
   * @return string
   *   The php image type to be saved.
   */
  protected function getType() {
    return $this->getPluginDefinition()['type'];
  }

}
